@@ -1,10 +1,9 @@
-
 #ifndef __SYMM_GROUPS__
 #define __SYMM_GROUPS__
 
 #include <vector>
 #include <utility>
 
-typedef std::vector<std::pair<std::vector<int>, std::vector<int> > > symmGroups;
+typedef std::vector<std::vector<std::pair<int, int> > > symmGroups;
 
 #endif
